# -*- coding: utf-8 -*-
# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

from tensorboard.compat.proto import summary_pb2

from tensorboard.plugins.npmi import plugin_data_pb2

PLUGIN_NAME = "npmi"

# The most recent value for the `version` field of the
# `ScalarPluginData` proto.
PROTO_VERSION = 0

ANNOTATIONS_TAG = "_npmi_/annotations"
METRICS_TAG = "_npmi_/metrics"
VALUES_TAG = "_npmi_/values"
EMBEDDINGS_TAG = "_npmi_/embeddings"


def create_summary_metadata(description):
    content = plugin_data_pb2.NpmiPluginData(version=PROTO_VERSION)
    return summary_pb2.SummaryMetadata(
        summary_description=description,
        plugin_data=summary_pb2.SummaryMetadata.PluginData(
            plugin_name=PLUGIN_NAME,
            content=content.SerializeToString(),
        ),
        data_class=summary_pb2.DATA_CLASS_TENSOR,
    )


def parse_plugin_metadata(content):
    """Parse summary metadata to a Python object.
    Arguments:
      content: The `content` field of a `SummaryMetadata` proto
        corresponding to the scalar plugin.
    Returns:
      A `ScalarPluginData` protobuf object.
    """
    if not isinstance(content, bytes):
        raise TypeError("Content type must be bytes")
    result = plugin_data_pb2.NpmiPluginData.FromString(content)
    if result.version == 0:
        return result
    # No other versions known at this time, so no migrations to do.
    return result



# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Compatibility interfaces for TensorBoard.

This module provides logic for importing variations on the TensorFlow
APIs, as lazily loaded imports to help avoid circular dependency issues
and defer the search and loading of the module until necessary.
"""


import tensorboard.lazy as _lazy


@_lazy.lazy_load("tensorboard.compat.tf")
def tf():
    """Provide the root module of a TF-like API for use within TensorBoard.

    By default this is equivalent to `import tensorflow as tf`, but it can be used
    in combination with //tensorboard/compat:tensorflow (to fall back to a stub TF
    API implementation if the real one is not available) or with
    //tensorboard/compat:no_tensorflow (to force unconditional use of the stub).

    Returns:
      The root module of a TF-like API, if available.

    Raises:
      ImportError: if a TF-like API is not available.
    """
    try:
        from tensorboard.compat import notf  # noqa: F401
    except ImportError:
        try:
            import tensorflow

            return tensorflow
        except ImportError:
            pass
    from tensorboard.compat import tensorflow_stub

    return tensorflow_stub


@_lazy.lazy_load("tensorboard.compat.tf2")
def tf2():
    """Provide the root module of a TF-2.0 API for use within TensorBoard.

    Returns:
      The root module of a TF-2.0 API, if available.

    Raises:
      ImportError: if a TF-2.0 API is not available.
    """
    # Resolve the lazy `tf` compat API from earlier in this file and try to find
    # tf.compat.v2. Don't check tf.__version__ since this is not always reliable
    # if TF was built with tf_api_version!=2.
    if hasattr(tf, "compat") and hasattr(tf.compat, "v2"):
        return tf.compat.v2
    raise ImportError("cannot import tensorflow 2.0 API")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/experiment.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorboard/uploader/proto/experiment.proto',
  package='tensorboard.service',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n+tensorboard/uploader/proto/experiment.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa6\x02\n\nExperiment\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12/\n\x0b\x63reate_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0bupdate_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0bnum_scalars\x18\x04 \x01(\x03\x12\x10\n\x08num_runs\x18\x05 \x01(\x03\x12\x10\n\x08num_tags\x18\x06 \x01(\x03\x12\x0c\n\x04name\x18\x07 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\x12\x1a\n\x12total_tensor_bytes\x18\t \x01(\x03\x12\x18\n\x10total_blob_bytes\x18\n \x01(\x03\x12\r\n\x05owner\x18\x0b \x01(\t\"\xf0\x01\n\x0e\x45xperimentMask\x12\x13\n\x0b\x63reate_time\x18\x02 \x01(\x08\x12\x13\n\x0bupdate_time\x18\x03 \x01(\x08\x12\x13\n\x0bnum_scalars\x18\x04 \x01(\x08\x12\x10\n\x08num_runs\x18\x05 \x01(\x08\x12\x10\n\x08num_tags\x18\x06 \x01(\x08\x12\x0c\n\x04name\x18\x07 \x01(\x08\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\x08\x12\x1a\n\x12total_tensor_bytes\x18\t \x01(\x08\x12\x18\n\x10total_blob_bytes\x18\n \x01(\x08\x12\r\n\x05owner\x18\x0b \x01(\x08J\x04\x08\x01\x10\x02R\rexperiment_idb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_EXPERIMENT = _descriptor.Descriptor(
  name='Experiment',
  full_name='tensorboard.service.Experiment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.Experiment.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='create_time', full_name='tensorboard.service.Experiment.create_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='tensorboard.service.Experiment.update_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_scalars', full_name='tensorboard.service.Experiment.num_scalars', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_runs', full_name='tensorboard.service.Experiment.num_runs', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_tags', full_name='tensorboard.service.Experiment.num_tags', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.Experiment.name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='tensorboard.service.Experiment.description', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_tensor_bytes', full_name='tensorboard.service.Experiment.total_tensor_bytes', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_blob_bytes', full_name='tensorboard.service.Experiment.total_blob_bytes', index=9,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner', full_name='tensorboard.service.Experiment.owner', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=396,
)


_EXPERIMENTMASK = _descriptor.Descriptor(
  name='ExperimentMask',
  full_name='tensorboard.service.ExperimentMask',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='create_time', full_name='tensorboard.service.ExperimentMask.create_time', index=0,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='tensorboard.service.ExperimentMask.update_time', index=1,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_scalars', full_name='tensorboard.service.ExperimentMask.num_scalars', index=2,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_runs', full_name='tensorboard.service.ExperimentMask.num_runs', index=3,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_tags', full_name='tensorboard.service.ExperimentMask.num_tags', index=4,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.ExperimentMask.name', index=5,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='tensorboard.service.ExperimentMask.description', index=6,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_tensor_bytes', full_name='tensorboard.service.ExperimentMask.total_tensor_bytes', index=7,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_blob_bytes', full_name='tensorboard.service.ExperimentMask.total_blob_bytes', index=8,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='owner', full_name='tensorboard.service.ExperimentMask.owner', index=9,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=399,
  serialized_end=639,
)

_EXPERIMENT.fields_by_name['create_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPERIMENT.fields_by_name['update_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['Experiment'] = _EXPERIMENT
DESCRIPTOR.message_types_by_name['ExperimentMask'] = _EXPERIMENTMASK
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Experiment = _reflection.GeneratedProtocolMessageType('Experiment', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENT,
  '__module__' : 'tensorboard.uploader.proto.experiment_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.Experiment)
  })
_sym_db.RegisterMessage(Experiment)

ExperimentMask = _reflection.GeneratedProtocolMessageType('ExperimentMask', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTMASK,
  '__module__' : 'tensorboard.uploader.proto.experiment_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.ExperimentMask)
  })
_sym_db.RegisterMessage(ExperimentMask)


# @@protoc_insertion_point(module_scope)



# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Experimental plugin support for TensorBoard.

Contains the mechanism for marking plugins as experimental.
"""


class ExperimentalPlugin(object):
    """A marker class used to annotate a plugin as experimental.

    Experimental plugins are hidden from users by default. The plugin will only
    be enabled for a user if the user has specified the plugin with the
    experimentalPlugin query parameter in the URL.

    The marker class can annotate either TBPlugin or TBLoader instances, whichever
    is most convenient.

    Typical usage is to create a new class that inherits from both an existing
    TBPlugin/TBLoader class and this marker class. For example:

    class ExperimentalGraphsPlugin(
        graphs_plugin.GraphsPlugin,
        experimental_plugin.ExperimentalPlugin,
    ):
        pass


    class ExperimentalDebuggerPluginLoader(
        debugger_plugin_loader.DebuggerPluginLoader,
        experimental_plugin.ExperimentalPlugin
    ):
        pass

    Note: This class is itself an experimental mechanism and is subject to
    modification or removal without warning.
    """

    pass



# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""The nPMI visualization plugin."""


import math
from werkzeug import wrappers

from tensorboard import plugin_util
from tensorboard.plugins import base_plugin
from tensorboard.backend import http_util
from tensorboard.data import provider

from tensorboard.plugins.npmi import metadata

_DEFAULT_DOWNSAMPLING = 1  # nPMI tensors per time series


def _error_response(request, error_message):
    return http_util.Respond(
        request,
        {"error": error_message},
        "application/json",
        code=400,
    )


def _missing_run_error_response(request):
    return _error_response(request, "run parameter is not provided")


# Convert all NaNs in a multidimensional list to None
def convert_nan_none(arr):
    return [
        convert_nan_none(e)
        if isinstance(e, list)
        else None
        if math.isnan(e)
        else e
        for e in arr
    ]


class NpmiPlugin(base_plugin.TBPlugin):
    """nPMI Plugin for Tensorboard."""

    plugin_name = metadata.PLUGIN_NAME

    def __init__(self, context):
        """Instantiates the nPMI Plugin via Tensorboard core.

        Args:
            context: A base_plugin.TBContext instance.
        """
        super(NpmiPlugin, self).__init__(context)
        self._logdir = context.logdir
        self._downsample_to = (context.sampling_hints or {}).get(
            self.plugin_name, _DEFAULT_DOWNSAMPLING
        )
        self._data_provider = context.data_provider
        self._version_checker = plugin_util._MetadataVersionChecker(
            data_kind="nPMI",
            latest_known_version=0,
        )

    def get_plugin_apps(self):
        return {
            "/tags": self.serve_tags,
            "/annotations": self.serve_annotations,
            "/metrics": self.serve_metrics,
            "/values": self.serve_values,
            "/embeddings": self.serve_embeddings,
        }

    def is_active(self):
        """Determines whether this plugin is active.

        This plugin is only active if TensorBoard sampled any npmi summaries.

        Returns:
          Whether this plugin is active.
        """
        return False  # `list_plugins` as called by TB core suffices

    def frontend_metadata(self):
        return base_plugin.FrontendMetadata(
            is_ng_component=True, tab_name="npmi", disable_reload=True
        )

    def tags_impl(self, ctx, experiment):
        mapping = self._data_provider.list_tensors(
            ctx, experiment_id=experiment, plugin_name=self.plugin_name
        )
        result = {run: {} for run in mapping}
        for (run, tag_to_content) in mapping.items():
            result[run] = []
            for (tag, metadatum) in tag_to_content.items():
                md = metadata.parse_plugin_metadata(metadatum.plugin_content)
                if not self._version_checker.ok(md.version, run, tag):
                    continue
                content = metadata.parse_plugin_metadata(
                    metadatum.plugin_content
                )
                result[run].append(tag)
        return result

    def annotations_impl(self, ctx, experiment):
        mapping = self._data_provider.list_tensors(
            ctx,
            experiment_id=experiment,
            plugin_name=self.plugin_name,
            run_tag_filter=provider.RunTagFilter(
                tags=[metadata.ANNOTATIONS_TAG]
            ),
        )
        result = {run: {} for run in mapping}
        for (run, _) in mapping.items():
            all_annotations = self._data_provider.read_tensors(
                ctx,
                experiment_id=experiment,
                plugin_name=self.plugin_name,
                run_tag_filter=provider.RunTagFilter(
                    runs=[run], tags=[metadata.ANNOTATIONS_TAG]
                ),
                downsample=self._downsample_to,
            )
            annotations = all_annotations.get(run, {}).get(
                metadata.ANNOTATIONS_TAG, {}
            )
            event_data = [
                annotation.decode("utf-8")
                for annotation in annotations[0].numpy
            ]
            result[run] = event_data
        return result

    def metrics_impl(self, ctx, experiment):
        mapping = self._data_provider.list_tensors(
            ctx,
            experiment_id=experiment,
            plugin_name=self.plugin_name,
            run_tag_filter=provider.RunTagFilter(tags=[metadata.METRICS_TAG]),
        )
        result = {run: {} for run in mapping}
        for (run, _) in mapping.items():
            all_metrics = self._data_provider.read_tensors(
                ctx,
                experiment_id=experiment,
                plugin_name=self.plugin_name,
                run_tag_filter=provider.RunTagFilter(
                    runs=[run], tags=[metadata.METRICS_TAG]
                ),
                downsample=self._downsample_to,
            )
            metrics = all_metrics.get(run, {}).get(metadata.METRICS_TAG, {})
            event_data = [metric.decode("utf-8") for metric in metrics[0].numpy]
            result[run] = event_data
        return result

    def values_impl(self, ctx, experiment):
        mapping = self._data_provider.list_tensors(
            ctx,
            experiment_id=experiment,
            plugin_name=self.plugin_name,
            run_tag_filter=provider.RunTagFilter(tags=[metadata.VALUES_TAG]),
        )
        result = {run: {} for run in mapping}
        for (run, _) in mapping.items():
            all_values = self._data_provider.read_tensors(
                ctx,
                experiment_id=experiment,
                plugin_name=self.plugin_name,
                run_tag_filter=provider.RunTagFilter(
                    runs=[run], tags=[metadata.VALUES_TAG]
                ),
                downsample=self._downsample_to,
            )
            values = all_values.get(run, {}).get(metadata.VALUES_TAG, {})
            event_data = values[0].numpy.tolist()
            event_data = convert_nan_none(event_data)
            result[run] = event_data
        return result

    def embeddings_impl(self, ctx, experiment):
        mapping = self._data_provider.list_tensors(
            ctx,
            experiment_id=experiment,
            plugin_name=self.plugin_name,
            run_tag_filter=provider.RunTagFilter(
                tags=[metadata.EMBEDDINGS_TAG]
            ),
        )
        result = {run: {} for run in mapping}
        for (run, _) in mapping.items():
            all_embeddings = self._data_provider.read_tensors(
                ctx,
                experiment_id=experiment,
                plugin_name=self.plugin_name,
                run_tag_filter=provider.RunTagFilter(
                    runs=[run], tags=[metadata.EMBEDDINGS_TAG]
                ),
                downsample=self._downsample_to,
            )
            embeddings = all_embeddings.get(run, {}).get(
                metadata.EMBEDDINGS_TAG, {}
            )
            event_data = embeddings[0].numpy.tolist()
            result[run] = event_data
        return result

    @wrappers.Request.application
    def serve_tags(self, request):
        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        contents = self.tags_impl(ctx, experiment=experiment)
        return http_util.Respond(request, contents, "application/json")

    @wrappers.Request.application
    def serve_annotations(self, request):
        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        contents = self.annotations_impl(ctx, experiment=experiment)
        return http_util.Respond(request, contents, "application/json")

    @wrappers.Request.application
    def serve_metrics(self, request):
        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        contents = self.metrics_impl(ctx, experiment=experiment)
        return http_util.Respond(request, contents, "application/json")

    @wrappers.Request.application
    def serve_values(self, request):
        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        contents = self.values_impl(ctx, experiment=experiment)
        return http_util.Respond(request, contents, "application/json")

    @wrappers.Request.application
    def serve_embeddings(self, request):
        ctx = plugin_util.context(request.environ)
        experiment = plugin_util.experiment_id(request.environ)
        contents = self.embeddings_impl(ctx, experiment=experiment)
        return http_util.Respond(request, contents, "application/json")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/tensor.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2
from tensorboard.compat.proto import tensor_pb2 as tensorboard_dot_compat_dot_proto_dot_tensor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorboard/uploader/proto/tensor.proto',
  package='tensorboard.service',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\'tensorboard/uploader/proto/tensor.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\x1a&tensorboard/compat/proto/summary.proto\x1a%tensorboard/compat/proto/tensor.proto\"s\n\x0bTensorPoint\x12\x0c\n\x04step\x18\x01 \x01(\x03\x12-\n\twall_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\'\n\x05value\x18\x03 \x01(\x0b\x32\x18.tensorboard.TensorProto\"\x88\x01\n\tTensorTag\x12\x10\n\x08max_step\x18\x01 \x01(\x03\x12\x31\n\rmax_wall_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x10summary_metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadatab\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tensorboard_dot_compat_dot_proto_dot_summary__pb2.DESCRIPTOR,tensorboard_dot_compat_dot_proto_dot_tensor__pb2.DESCRIPTOR,])




_TENSORPOINT = _descriptor.Descriptor(
  name='TensorPoint',
  full_name='tensorboard.service.TensorPoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='step', full_name='tensorboard.service.TensorPoint.step', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wall_time', full_name='tensorboard.service.TensorPoint.wall_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='tensorboard.service.TensorPoint.value', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=176,
  serialized_end=291,
)


_TENSORTAG = _descriptor.Descriptor(
  name='TensorTag',
  full_name='tensorboard.service.TensorTag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='max_step', full_name='tensorboard.service.TensorTag.max_step', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_wall_time', full_name='tensorboard.service.TensorTag.max_wall_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='summary_metadata', full_name='tensorboard.service.TensorTag.summary_metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=294,
  serialized_end=430,
)

_TENSORPOINT.fields_by_name['wall_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TENSORPOINT.fields_by_name['value'].message_type = tensorboard_dot_compat_dot_proto_dot_tensor__pb2._TENSORPROTO
_TENSORTAG.fields_by_name['max_wall_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TENSORTAG.fields_by_name['summary_metadata'].message_type = tensorboard_dot_compat_dot_proto_dot_summary__pb2._SUMMARYMETADATA
DESCRIPTOR.message_types_by_name['TensorPoint'] = _TENSORPOINT
DESCRIPTOR.message_types_by_name['TensorTag'] = _TENSORTAG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TensorPoint = _reflection.GeneratedProtocolMessageType('TensorPoint', (_message.Message,), {
  'DESCRIPTOR' : _TENSORPOINT,
  '__module__' : 'tensorboard.uploader.proto.tensor_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.TensorPoint)
  })
_sym_db.RegisterMessage(TensorPoint)

TensorTag = _reflection.GeneratedProtocolMessageType('TensorTag', (_message.Message,), {
  'DESCRIPTOR' : _TENSORTAG,
  '__module__' : 'tensorboard.uploader.proto.tensor_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.TensorTag)
  })
_sym_db.RegisterMessage(TensorTag)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/write_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorboard.uploader.proto import experiment_pb2 as tensorboard_dot_uploader_dot_proto_dot_experiment__pb2
from tensorboard.uploader.proto import scalar_pb2 as tensorboard_dot_uploader_dot_proto_dot_scalar__pb2
from tensorboard.uploader.proto import tensor_pb2 as tensorboard_dot_uploader_dot_proto_dot_tensor__pb2
from tensorboard.uploader.proto import blob_pb2 as tensorboard_dot_uploader_dot_proto_dot_blob__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorboard/uploader/proto/write_service.proto',
  package='tensorboard.service',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n.tensorboard/uploader/proto/write_service.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\x1a+tensorboard/uploader/proto/experiment.proto\x1a\'tensorboard/uploader/proto/scalar.proto\x1a\'tensorboard/uploader/proto/tensor.proto\x1a%tensorboard/uploader/proto/blob.proto\x1a&tensorboard/compat/proto/summary.proto\"<\n\x17\x43reateExperimentRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\">\n\x18\x43reateExperimentResponse\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\"\x8c\x01\n\x17UpdateExperimentRequest\x12\x33\n\nexperiment\x18\x01 \x01(\x0b\x32\x1f.tensorboard.service.Experiment\x12<\n\x0f\x65xperiment_mask\x18\x02 \x01(\x0b\x32#.tensorboard.service.ExperimentMask\"\x1a\n\x18UpdateExperimentResponse\"0\n\x17\x44\x65leteExperimentRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\"\x1a\n\x18\x44\x65leteExperimentResponse\"\'\n\x10PurgeDataRequest\x12\x13\n\x0b\x62\x61tch_limit\x18\x01 \x01(\x05\"I\n\x11PurgeDataResponse\x12\x34\n\x0bpurge_stats\x18\x01 \x01(\x0b\x32\x1f.tensorboard.service.PurgeStats\">\n\nPurgeStats\x12\x0c\n\x04tags\x18\x01 \x01(\x05\x12\x13\n\x0b\x65xperiments\x18\x02 \x01(\x05\x12\r\n\x05users\x18\x03 \x01(\x05\"\xad\x02\n\x12WriteScalarRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x39\n\x04runs\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteScalarRequest.Run\x1aN\n\x03Run\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x04tags\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteScalarRequest.Tag\x1au\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x06points\x18\x02 \x03(\x0b\x32 .tensorboard.service.ScalarPoint\x12.\n\x08metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\"\x15\n\x13WriteScalarResponse\"\xad\x02\n\x12WriteTensorRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x39\n\x04runs\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteTensorRequest.Run\x1aN\n\x03Run\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x04tags\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteTensorRequest.Tag\x1au\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x06points\x18\x02 \x03(\x0b\x32 .tensorboard.service.TensorPoint\x12.\n\x08metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\"\x15\n\x13WriteTensorResponse\"\xdd\x01\n\x1eGetOrCreateBlobSequenceRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0b\n\x03run\x18\x02 \x01(\t\x12\x0b\n\x03tag\x18\x03 \x01(\t\x12\x0c\n\x04step\x18\x04 \x01(\x03\x12-\n\twall_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1d\n\x15\x66inal_sequence_length\x18\x06 \x01(\x03\x12.\n\x08metadata\x18\x07 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\";\n\x1fGetOrCreateBlobSequenceResponse\x12\x18\n\x10\x62lob_sequence_id\x18\x01 \x01(\t\"A\n\x16GetBlobMetadataRequest\x12\x18\n\x10\x62lob_sequence_id\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\x03\"k\n\x17GetBlobMetadataResponse\x12\x32\n\nblob_state\x18\x01 \x01(\x0e\x32\x1e.tensorboard.service.BlobState\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0e\n\x06\x63rc32c\x18\x03 \x01(\x07\"\xac\x01\n\x10WriteBlobRequest\x12\x18\n\x10\x62lob_sequence_id\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x0e\n\x06offset\x18\x04 \x01(\x03\x12\x0e\n\x06\x63rc32c\x18\x05 \x01(\x07\x12\x17\n\x0f\x66inalize_object\x18\x06 \x01(\x08\x12\x14\n\x0c\x66inal_crc32c\x18\x07 \x01(\x07\x12\x12\n\nblob_bytes\x18\x08 \x01(\x03\"U\n\x11WriteBlobResponse\x12\x32\n\nblob_state\x18\x01 \x01(\x0e\x32\x1e.tensorboard.service.BlobState\x12\x0c\n\x04size\x18\x02 \x01(\x03\"\x16\n\x14\x44\x65leteOwnUserRequest\"\x17\n\x15\x44\x65leteOwnUserResponse2\xde\x08\n\x18TensorBoardWriterService\x12q\n\x10\x43reateExperiment\x12,.tensorboard.service.CreateExperimentRequest\x1a-.tensorboard.service.CreateExperimentResponse\"\x00\x12q\n\x10UpdateExperiment\x12,.tensorboard.service.UpdateExperimentRequest\x1a-.tensorboard.service.UpdateExperimentResponse\"\x00\x12q\n\x10\x44\x65leteExperiment\x12,.tensorboard.service.DeleteExperimentRequest\x1a-.tensorboard.service.DeleteExperimentResponse\"\x00\x12\\\n\tPurgeData\x12%.tensorboard.service.PurgeDataRequest\x1a&.tensorboard.service.PurgeDataResponse\"\x00\x12\x62\n\x0bWriteScalar\x12\'.tensorboard.service.WriteScalarRequest\x1a(.tensorboard.service.WriteScalarResponse\"\x00\x12\x62\n\x0bWriteTensor\x12\'.tensorboard.service.WriteTensorRequest\x1a(.tensorboard.service.WriteTensorResponse\"\x00\x12\x86\x01\n\x17GetOrCreateBlobSequence\x12\x33.tensorboard.service.GetOrCreateBlobSequenceRequest\x1a\x34.tensorboard.service.GetOrCreateBlobSequenceResponse\"\x00\x12n\n\x0fGetBlobMetadata\x12+.tensorboard.service.GetBlobMetadataRequest\x1a,.tensorboard.service.GetBlobMetadataResponse\"\x00\x12`\n\tWriteBlob\x12%.tensorboard.service.WriteBlobRequest\x1a&.tensorboard.service.WriteBlobResponse\"\x00(\x01\x30\x01\x12h\n\rDeleteOwnUser\x12).tensorboard.service.DeleteOwnUserRequest\x1a*.tensorboard.service.DeleteOwnUserResponse\"\x00\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tensorboard_dot_uploader_dot_proto_dot_experiment__pb2.DESCRIPTOR,tensorboard_dot_uploader_dot_proto_dot_scalar__pb2.DESCRIPTOR,tensorboard_dot_uploader_dot_proto_dot_tensor__pb2.DESCRIPTOR,tensorboard_dot_uploader_dot_proto_dot_blob__pb2.DESCRIPTOR,tensorboard_dot_compat_dot_proto_dot_summary__pb2.DESCRIPTOR,])




_CREATEEXPERIMENTREQUEST = _descriptor.Descriptor(
  name='CreateExperimentRequest',
  full_name='tensorboard.service.CreateExperimentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.CreateExperimentRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='tensorboard.service.CreateExperimentRequest.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=310,
  serialized_end=370,
)


_CREATEEXPERIMENTRESPONSE = _descriptor.Descriptor(
  name='CreateExperimentResponse',
  full_name='tensorboard.service.CreateExperimentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.CreateExperimentResponse.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='url', full_name='tensorboard.service.CreateExperimentResponse.url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=372,
  serialized_end=434,
)


_UPDATEEXPERIMENTREQUEST = _descriptor.Descriptor(
  name='UpdateExperimentRequest',
  full_name='tensorboard.service.UpdateExperimentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment', full_name='tensorboard.service.UpdateExperimentRequest.experiment', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='experiment_mask', full_name='tensorboard.service.UpdateExperimentRequest.experiment_mask', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=437,
  serialized_end=577,
)


_UPDATEEXPERIMENTRESPONSE = _descriptor.Descriptor(
  name='UpdateExperimentResponse',
  full_name='tensorboard.service.UpdateExperimentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=579,
  serialized_end=605,
)


_DELETEEXPERIMENTREQUEST = _descriptor.Descriptor(
  name='DeleteExperimentRequest',
  full_name='tensorboard.service.DeleteExperimentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.DeleteExperimentRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=607,
  serialized_end=655,
)


_DELETEEXPERIMENTRESPONSE = _descriptor.Descriptor(
  name='DeleteExperimentResponse',
  full_name='tensorboard.service.DeleteExperimentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=657,
  serialized_end=683,
)


_PURGEDATAREQUEST = _descriptor.Descriptor(
  name='PurgeDataRequest',
  full_name='tensorboard.service.PurgeDataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batch_limit', full_name='tensorboard.service.PurgeDataRequest.batch_limit', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=685,
  serialized_end=724,
)


_PURGEDATARESPONSE = _descriptor.Descriptor(
  name='PurgeDataResponse',
  full_name='tensorboard.service.PurgeDataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='purge_stats', full_name='tensorboard.service.PurgeDataResponse.purge_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=726,
  serialized_end=799,
)


_PURGESTATS = _descriptor.Descriptor(
  name='PurgeStats',
  full_name='tensorboard.service.PurgeStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tags', full_name='tensorboard.service.PurgeStats.tags', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='experiments', full_name='tensorboard.service.PurgeStats.experiments', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='users', full_name='tensorboard.service.PurgeStats.users', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=801,
  serialized_end=863,
)


_WRITESCALARREQUEST_RUN = _descriptor.Descriptor(
  name='Run',
  full_name='tensorboard.service.WriteScalarRequest.Run',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.WriteScalarRequest.Run.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='tensorboard.service.WriteScalarRequest.Run.tags', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=970,
  serialized_end=1048,
)

_WRITESCALARREQUEST_TAG = _descriptor.Descriptor(
  name='Tag',
  full_name='tensorboard.service.WriteScalarRequest.Tag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.WriteScalarRequest.Tag.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='points', full_name='tensorboard.service.WriteScalarRequest.Tag.points', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tensorboard.service.WriteScalarRequest.Tag.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1050,
  serialized_end=1167,
)

_WRITESCALARREQUEST = _descriptor.Descriptor(
  name='WriteScalarRequest',
  full_name='tensorboard.service.WriteScalarRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.WriteScalarRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runs', full_name='tensorboard.service.WriteScalarRequest.runs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_WRITESCALARREQUEST_RUN, _WRITESCALARREQUEST_TAG, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=866,
  serialized_end=1167,
)


_WRITESCALARRESPONSE = _descriptor.Descriptor(
  name='WriteScalarResponse',
  full_name='tensorboard.service.WriteScalarResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1169,
  serialized_end=1190,
)


_WRITETENSORREQUEST_RUN = _descriptor.Descriptor(
  name='Run',
  full_name='tensorboard.service.WriteTensorRequest.Run',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.WriteTensorRequest.Run.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='tensorboard.service.WriteTensorRequest.Run.tags', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1297,
  serialized_end=1375,
)

_WRITETENSORREQUEST_TAG = _descriptor.Descriptor(
  name='Tag',
  full_name='tensorboard.service.WriteTensorRequest.Tag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.WriteTensorRequest.Tag.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='points', full_name='tensorboard.service.WriteTensorRequest.Tag.points', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tensorboard.service.WriteTensorRequest.Tag.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1377,
  serialized_end=1494,
)

_WRITETENSORREQUEST = _descriptor.Descriptor(
  name='WriteTensorRequest',
  full_name='tensorboard.service.WriteTensorRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.WriteTensorRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runs', full_name='tensorboard.service.WriteTensorRequest.runs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_WRITETENSORREQUEST_RUN, _WRITETENSORREQUEST_TAG, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1193,
  serialized_end=1494,
)


_WRITETENSORRESPONSE = _descriptor.Descriptor(
  name='WriteTensorResponse',
  full_name='tensorboard.service.WriteTensorResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1496,
  serialized_end=1517,
)


_GETORCREATEBLOBSEQUENCEREQUEST = _descriptor.Descriptor(
  name='GetOrCreateBlobSequenceRequest',
  full_name='tensorboard.service.GetOrCreateBlobSequenceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.run', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tag', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.tag', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='step', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.step', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wall_time', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.wall_time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='final_sequence_length', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.final_sequence_length', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tensorboard.service.GetOrCreateBlobSequenceRequest.metadata', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1520,
  serialized_end=1741,
)


_GETORCREATEBLOBSEQUENCERESPONSE = _descriptor.Descriptor(
  name='GetOrCreateBlobSequenceResponse',
  full_name='tensorboard.service.GetOrCreateBlobSequenceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blob_sequence_id', full_name='tensorboard.service.GetOrCreateBlobSequenceResponse.blob_sequence_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1743,
  serialized_end=1802,
)


_GETBLOBMETADATAREQUEST = _descriptor.Descriptor(
  name='GetBlobMetadataRequest',
  full_name='tensorboard.service.GetBlobMetadataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blob_sequence_id', full_name='tensorboard.service.GetBlobMetadataRequest.blob_sequence_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='tensorboard.service.GetBlobMetadataRequest.index', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1804,
  serialized_end=1869,
)


_GETBLOBMETADATARESPONSE = _descriptor.Descriptor(
  name='GetBlobMetadataResponse',
  full_name='tensorboard.service.GetBlobMetadataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blob_state', full_name='tensorboard.service.GetBlobMetadataResponse.blob_state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='tensorboard.service.GetBlobMetadataResponse.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='crc32c', full_name='tensorboard.service.GetBlobMetadataResponse.crc32c', index=2,
      number=3, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1871,
  serialized_end=1978,
)


_WRITEBLOBREQUEST = _descriptor.Descriptor(
  name='WriteBlobRequest',
  full_name='tensorboard.service.WriteBlobRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blob_sequence_id', full_name='tensorboard.service.WriteBlobRequest.blob_sequence_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='index', full_name='tensorboard.service.WriteBlobRequest.index', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='tensorboard.service.WriteBlobRequest.data', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='tensorboard.service.WriteBlobRequest.offset', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='crc32c', full_name='tensorboard.service.WriteBlobRequest.crc32c', index=4,
      number=5, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='finalize_object', full_name='tensorboard.service.WriteBlobRequest.finalize_object', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='final_crc32c', full_name='tensorboard.service.WriteBlobRequest.final_crc32c', index=6,
      number=7, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blob_bytes', full_name='tensorboard.service.WriteBlobRequest.blob_bytes', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1981,
  serialized_end=2153,
)


_WRITEBLOBRESPONSE = _descriptor.Descriptor(
  name='WriteBlobResponse',
  full_name='tensorboard.service.WriteBlobResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blob_state', full_name='tensorboard.service.WriteBlobResponse.blob_state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='tensorboard.service.WriteBlobResponse.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2155,
  serialized_end=2240,
)


_DELETEOWNUSERREQUEST = _descriptor.Descriptor(
  name='DeleteOwnUserRequest',
  full_name='tensorboard.service.DeleteOwnUserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2242,
  serialized_end=2264,
)


_DELETEOWNUSERRESPONSE = _descriptor.Descriptor(
  name='DeleteOwnUserResponse',
  full_name='tensorboard.service.DeleteOwnUserResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2266,
  serialized_end=2289,
)

_UPDATEEXPERIMENTREQUEST.fields_by_name['experiment'].message_type = tensorboard_dot_uploader_dot_proto_dot_experiment__pb2._EXPERIMENT
_UPDATEEXPERIMENTREQUEST.fields_by_name['experiment_mask'].message_type = tensorboard_dot_uploader_dot_proto_dot_experiment__pb2._EXPERIMENTMASK
_PURGEDATARESPONSE.fields_by_name['purge_stats'].message_type = _PURGESTATS
_WRITESCALARREQUEST_RUN.fields_by_name['tags'].message_type = _WRITESCALARREQUEST_TAG
_WRITESCALARREQUEST_RUN.containing_type = _WRITESCALARREQUEST
_WRITESCALARREQUEST_TAG.fields_by_name['points'].message_type = tensorboard_dot_uploader_dot_proto_dot_scalar__pb2._SCALARPOINT
_WRITESCALARREQUEST_TAG.fields_by_name['metadata'].message_type = tensorboard_dot_compat_dot_proto_dot_summary__pb2._SUMMARYMETADATA
_WRITESCALARREQUEST_TAG.containing_type = _WRITESCALARREQUEST
_WRITESCALARREQUEST.fields_by_name['runs'].message_type = _WRITESCALARREQUEST_RUN
_WRITETENSORREQUEST_RUN.fields_by_name['tags'].message_type = _WRITETENSORREQUEST_TAG
_WRITETENSORREQUEST_RUN.containing_type = _WRITETENSORREQUEST
_WRITETENSORREQUEST_TAG.fields_by_name['points'].message_type = tensorboard_dot_uploader_dot_proto_dot_tensor__pb2._TENSORPOINT
_WRITETENSORREQUEST_TAG.fields_by_name['metadata'].message_type = tensorboard_dot_compat_dot_proto_dot_summary__pb2._SUMMARYMETADATA
_WRITETENSORREQUEST_TAG.containing_type = _WRITETENSORREQUEST
_WRITETENSORREQUEST.fields_by_name['runs'].message_type = _WRITETENSORREQUEST_RUN
_GETORCREATEBLOBSEQUENCEREQUEST.fields_by_name['wall_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GETORCREATEBLOBSEQUENCEREQUEST.fields_by_name['metadata'].message_type = tensorboard_dot_compat_dot_proto_dot_summary__pb2._SUMMARYMETADATA
_GETBLOBMETADATARESPONSE.fields_by_name['blob_state'].enum_type = tensorboard_dot_uploader_dot_proto_dot_blob__pb2._BLOBSTATE
_WRITEBLOBRESPONSE.fields_by_name['blob_state'].enum_type = tensorboard_dot_uploader_dot_proto_dot_blob__pb2._BLOBSTATE
DESCRIPTOR.message_types_by_name['CreateExperimentRequest'] = _CREATEEXPERIMENTREQUEST
DESCRIPTOR.message_types_by_name['CreateExperimentResponse'] = _CREATEEXPERIMENTRESPONSE
DESCRIPTOR.message_types_by_name['UpdateExperimentRequest'] = _UPDATEEXPERIMENTREQUEST
DESCRIPTOR.message_types_by_name['UpdateExperimentResponse'] = _UPDATEEXPERIMENTRESPONSE
DESCRIPTOR.message_types_by_name['DeleteExperimentRequest'] = _DELETEEXPERIMENTREQUEST
DESCRIPTOR.message_types_by_name['DeleteExperimentResponse'] = _DELETEEXPERIMENTRESPONSE
DESCRIPTOR.message_types_by_name['PurgeDataRequest'] = _PURGEDATAREQUEST
DESCRIPTOR.message_types_by_name['PurgeDataResponse'] = _PURGEDATARESPONSE
DESCRIPTOR.message_types_by_name['PurgeStats'] = _PURGESTATS
DESCRIPTOR.message_types_by_name['WriteScalarRequest'] = _WRITESCALARREQUEST
DESCRIPTOR.message_types_by_name['WriteScalarResponse'] = _WRITESCALARRESPONSE
DESCRIPTOR.message_types_by_name['WriteTensorRequest'] = _WRITETENSORREQUEST
DESCRIPTOR.message_types_by_name['WriteTensorResponse'] = _WRITETENSORRESPONSE
DESCRIPTOR.message_types_by_name['GetOrCreateBlobSequenceRequest'] = _GETORCREATEBLOBSEQUENCEREQUEST
DESCRIPTOR.message_types_by_name['GetOrCreateBlobSequenceResponse'] = _GETORCREATEBLOBSEQUENCERESPONSE
DESCRIPTOR.message_types_by_name['GetBlobMetadataRequest'] = _GETBLOBMETADATAREQUEST
DESCRIPTOR.message_types_by_name['GetBlobMetadataResponse'] = _GETBLOBMETADATARESPONSE
DESCRIPTOR.message_types_by_name['WriteBlobRequest'] = _WRITEBLOBREQUEST
DESCRIPTOR.message_types_by_name['WriteBlobResponse'] = _WRITEBLOBRESPONSE
DESCRIPTOR.message_types_by_name['DeleteOwnUserRequest'] = _DELETEOWNUSERREQUEST
DESCRIPTOR.message_types_by_name['DeleteOwnUserResponse'] = _DELETEOWNUSERRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateExperimentRequest = _reflection.GeneratedProtocolMessageType('CreateExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.CreateExperimentRequest)
  })
_sym_db.RegisterMessage(CreateExperimentRequest)

CreateExperimentResponse = _reflection.GeneratedProtocolMessageType('CreateExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.CreateExperimentResponse)
  })
_sym_db.RegisterMessage(CreateExperimentResponse)

UpdateExperimentRequest = _reflection.GeneratedProtocolMessageType('UpdateExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.UpdateExperimentRequest)
  })
_sym_db.RegisterMessage(UpdateExperimentRequest)

UpdateExperimentResponse = _reflection.GeneratedProtocolMessageType('UpdateExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.UpdateExperimentResponse)
  })
_sym_db.RegisterMessage(UpdateExperimentResponse)

DeleteExperimentRequest = _reflection.GeneratedProtocolMessageType('DeleteExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteExperimentRequest)
  })
_sym_db.RegisterMessage(DeleteExperimentRequest)

DeleteExperimentResponse = _reflection.GeneratedProtocolMessageType('DeleteExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteExperimentResponse)
  })
_sym_db.RegisterMessage(DeleteExperimentResponse)

PurgeDataRequest = _reflection.GeneratedProtocolMessageType('PurgeDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _PURGEDATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeDataRequest)
  })
_sym_db.RegisterMessage(PurgeDataRequest)

PurgeDataResponse = _reflection.GeneratedProtocolMessageType('PurgeDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _PURGEDATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeDataResponse)
  })
_sym_db.RegisterMessage(PurgeDataResponse)

PurgeStats = _reflection.GeneratedProtocolMessageType('PurgeStats', (_message.Message,), {
  'DESCRIPTOR' : _PURGESTATS,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeStats)
  })
_sym_db.RegisterMessage(PurgeStats)

WriteScalarRequest = _reflection.GeneratedProtocolMessageType('WriteScalarRequest', (_message.Message,), {

  'Run' : _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
    'DESCRIPTOR' : _WRITESCALARREQUEST_RUN,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest.Run)
    })
  ,

  'Tag' : _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
    'DESCRIPTOR' : _WRITESCALARREQUEST_TAG,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest.Tag)
    })
  ,
  'DESCRIPTOR' : _WRITESCALARREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest)
  })
_sym_db.RegisterMessage(WriteScalarRequest)
_sym_db.RegisterMessage(WriteScalarRequest.Run)
_sym_db.RegisterMessage(WriteScalarRequest.Tag)

WriteScalarResponse = _reflection.GeneratedProtocolMessageType('WriteScalarResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITESCALARRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarResponse)
  })
_sym_db.RegisterMessage(WriteScalarResponse)

WriteTensorRequest = _reflection.GeneratedProtocolMessageType('WriteTensorRequest', (_message.Message,), {

  'Run' : _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
    'DESCRIPTOR' : _WRITETENSORREQUEST_RUN,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorRequest.Run)
    })
  ,

  'Tag' : _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
    'DESCRIPTOR' : _WRITETENSORREQUEST_TAG,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorRequest.Tag)
    })
  ,
  'DESCRIPTOR' : _WRITETENSORREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorRequest)
  })
_sym_db.RegisterMessage(WriteTensorRequest)
_sym_db.RegisterMessage(WriteTensorRequest.Run)
_sym_db.RegisterMessage(WriteTensorRequest.Tag)

WriteTensorResponse = _reflection.GeneratedProtocolMessageType('WriteTensorResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITETENSORRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteTensorResponse)
  })
_sym_db.RegisterMessage(WriteTensorResponse)

GetOrCreateBlobSequenceRequest = _reflection.GeneratedProtocolMessageType('GetOrCreateBlobSequenceRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETORCREATEBLOBSEQUENCEREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetOrCreateBlobSequenceRequest)
  })
_sym_db.RegisterMessage(GetOrCreateBlobSequenceRequest)

GetOrCreateBlobSequenceResponse = _reflection.GeneratedProtocolMessageType('GetOrCreateBlobSequenceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETORCREATEBLOBSEQUENCERESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetOrCreateBlobSequenceResponse)
  })
_sym_db.RegisterMessage(GetOrCreateBlobSequenceResponse)

GetBlobMetadataRequest = _reflection.GeneratedProtocolMessageType('GetBlobMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOBMETADATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetBlobMetadataRequest)
  })
_sym_db.RegisterMessage(GetBlobMetadataRequest)

GetBlobMetadataResponse = _reflection.GeneratedProtocolMessageType('GetBlobMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOBMETADATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.GetBlobMetadataResponse)
  })
_sym_db.RegisterMessage(GetBlobMetadataResponse)

WriteBlobRequest = _reflection.GeneratedProtocolMessageType('WriteBlobRequest', (_message.Message,), {
  'DESCRIPTOR' : _WRITEBLOBREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteBlobRequest)
  })
_sym_db.RegisterMessage(WriteBlobRequest)

WriteBlobResponse = _reflection.GeneratedProtocolMessageType('WriteBlobResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITEBLOBRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteBlobResponse)
  })
_sym_db.RegisterMessage(WriteBlobResponse)

DeleteOwnUserRequest = _reflection.GeneratedProtocolMessageType('DeleteOwnUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOWNUSERREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteOwnUserRequest)
  })
_sym_db.RegisterMessage(DeleteOwnUserRequest)

DeleteOwnUserResponse = _reflection.GeneratedProtocolMessageType('DeleteOwnUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOWNUSERRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteOwnUserResponse)
  })
_sym_db.RegisterMessage(DeleteOwnUserResponse)



_TENSORBOARDWRITERSERVICE = _descriptor.ServiceDescriptor(
  name='TensorBoardWriterService',
  full_name='tensorboard.service.TensorBoardWriterService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2292,
  serialized_end=3410,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateExperiment',
    full_name='tensorboard.service.TensorBoardWriterService.CreateExperiment',
    index=0,
    containing_service=None,
    input_type=_CREATEEXPERIMENTREQUEST,
    output_type=_CREATEEXPERIMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateExperiment',
    full_name='tensorboard.service.TensorBoardWriterService.UpdateExperiment',
    index=1,
    containing_service=None,
    input_type=_UPDATEEXPERIMENTREQUEST,
    output_type=_UPDATEEXPERIMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteExperiment',
    full_name='tensorboard.service.TensorBoardWriterService.DeleteExperiment',
    index=2,
    containing_service=None,
    input_type=_DELETEEXPERIMENTREQUEST,
    output_type=_DELETEEXPERIMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PurgeData',
    full_name='tensorboard.service.TensorBoardWriterService.PurgeData',
    index=3,
    containing_service=None,
    input_type=_PURGEDATAREQUEST,
    output_type=_PURGEDATARESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='WriteScalar',
    full_name='tensorboard.service.TensorBoardWriterService.WriteScalar',
    index=4,
    containing_service=None,
    input_type=_WRITESCALARREQUEST,
    output_type=_WRITESCALARRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='WriteTensor',
    full_name='tensorboard.service.TensorBoardWriterService.WriteTensor',
    index=5,
    containing_service=None,
    input_type=_WRITETENSORREQUEST,
    output_type=_WRITETENSORRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetOrCreateBlobSequence',
    full_name='tensorboard.service.TensorBoardWriterService.GetOrCreateBlobSequence',
    index=6,
    containing_service=None,
    input_type=_GETORCREATEBLOBSEQUENCEREQUEST,
    output_type=_GETORCREATEBLOBSEQUENCERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetBlobMetadata',
    full_name='tensorboard.service.TensorBoardWriterService.GetBlobMetadata',
    index=7,
    containing_service=None,
    input_type=_GETBLOBMETADATAREQUEST,
    output_type=_GETBLOBMETADATARESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='WriteBlob',
    full_name='tensorboard.service.TensorBoardWriterService.WriteBlob',
    index=8,
    containing_service=None,
    input_type=_WRITEBLOBREQUEST,
    output_type=_WRITEBLOBRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteOwnUser',
    full_name='tensorboard.service.TensorBoardWriterService.DeleteOwnUser',
    index=9,
    containing_service=None,
    input_type=_DELETEOWNUSERREQUEST,
    output_type=_DELETEOWNUSERRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TENSORBOARDWRITERSERVICE)

DESCRIPTOR.services_by_name['TensorBoardWriterService'] = _TENSORBOARDWRITERSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Writes events to disk in a logdir."""


import os
import queue
import socket
import threading
import time

from tensorboard.compat import tf
from tensorboard.compat.proto import event_pb2
from tensorboard.summary.writer.record_writer import RecordWriter


class AtomicCounter(object):
    def __init__(self, initial_value):
        self._value = initial_value
        self._lock = threading.Lock()

    def get(self):
        with self._lock:
            try:
                return self._value
            finally:
                self._value += 1


_global_uid = AtomicCounter(0)


class EventFileWriter(object):
    """Writes `Event` protocol buffers to an event file.

    The `EventFileWriter` class creates an event file in the specified
    directory, and asynchronously writes Event protocol buffers to the
    file. The Event file is encoded using the tfrecord format, which is
    similar to RecordIO.
    """

    def __init__(
        self, logdir, max_queue_size=10, flush_secs=120, filename_suffix=""
    ):
        """Creates a `EventFileWriter` and an event file to write to.

        On construction the summary writer creates a new event file in `logdir`.
        This event file will contain `Event` protocol buffers, which are written to
        disk via the add_event method.
        The other arguments to the constructor control the asynchronous writes to
        the event file:

        Args:
          logdir: A string. Directory where event file will be written.
          max_queue_size: Integer. Size of the queue for pending events and summaries.
          flush_secs: Number. How often, in seconds, to flush the
            pending events and summaries to disk.
        """
        self._logdir = logdir
        tf.io.gfile.makedirs(logdir)
        self._file_name = (
            os.path.join(
                logdir,
                "events.out.tfevents.%010d.%s.%s.%s"
                % (
                    time.time(),
                    socket.gethostname(),
                    os.getpid(),
                    _global_uid.get(),
                ),
            )
            + filename_suffix
        )  # noqa E128
        self._general_file_writer = tf.io.gfile.GFile(self._file_name, "wb")
        self._async_writer = _AsyncWriter(
            RecordWriter(self._general_file_writer), max_queue_size, flush_secs
        )

        # Initialize an event instance.
        _event = event_pb2.Event(
            wall_time=time.time(), file_version="brain.Event:2"
        )
        self.add_event(_event)
        self.flush()

    def get_logdir(self):
        """Returns the directory where event file will be written."""
        return self._logdir

    def add_event(self, event):
        """Adds an event to the event file.

        Args:
          event: An `Event` protocol buffer.
        """
        if not isinstance(event, event_pb2.Event):
            raise TypeError(
                "Expected an event_pb2.Event proto, "
                " but got %s" % type(event)
            )
        self._async_writer.write(event.SerializeToString())

    def flush(self):
        """Flushes the event file to disk.

        Call this method to make sure that all pending events have been
        written to disk.
        """
        self._async_writer.flush()

    def close(self):
        """Performs a final flush of the event file to disk, stops the
        write/flush worker and closes the file.

        Call this method when you do not need the summary writer
        anymore.
        """
        self._async_writer.close()


class _AsyncWriter(object):
    """Writes bytes to a file."""

    def __init__(self, record_writer, max_queue_size=20, flush_secs=120):
        """Writes bytes to a file asynchronously. An instance of this class
        holds a queue to keep the incoming data temporarily. Data passed to the
        `write` function will be put to the queue and the function returns
        immediately. This class also maintains a thread to write data in the
        queue to disk. The first initialization parameter is an instance of
        `tensorboard.summary.record_writer` which computes the CRC checksum and
        then write the combined result to the disk. So we use an async approach
        to improve performance.

        Args:
            record_writer: A RecordWriter instance
            max_queue_size: Integer. Size of the queue for pending bytestrings.
            flush_secs: Number. How often, in seconds, to flush the
                pending bytestrings to disk.
        """
        self._writer = record_writer
        self._closed = False
        self._byte_queue = queue.Queue(max_queue_size)
        self._worker = _AsyncWriterThread(
            self._byte_queue, self._writer, flush_secs
        )
        self._lock = threading.Lock()
        self._worker.start()

    def write(self, bytestring):
        """Enqueue the given bytes to be written asychronously."""
        with self._lock:
            if self._closed:
                raise IOError("Writer is closed")
            self._byte_queue.put(bytestring)

    def flush(self):
        """Write all the enqueued bytestring before this flush call to disk.

        Block until all the above bytestring are written.
        """
        with self._lock:
            if self._closed:
                raise IOError("Writer is closed")
            self._byte_queue.join()
            self._writer.flush()

    def close(self):
        """Closes the underlying writer, flushing any pending writes first."""
        if not self._closed:
            with self._lock:
                if not self._closed:
                    self._closed = True
                    self._worker.stop()
                    self._writer.flush()
                    self._writer.close()


class _AsyncWriterThread(threading.Thread):
    """Thread that processes asynchronous writes for _AsyncWriter."""

    def __init__(self, queue, record_writer, flush_secs):
        """Creates an _AsyncWriterThread.

        Args:
          queue: A Queue from which to dequeue data.
          record_writer: An instance of record_writer writer.
          flush_secs: How often, in seconds, to flush the
            pending file to disk.
        """
        threading.Thread.__init__(self)
        self.daemon = True
        self._queue = queue
        self._record_writer = record_writer
        self._flush_secs = flush_secs
        # The first data will be flushed immediately.
        self._next_flush_time = 0
        self._has_pending_data = False
        self._shutdown_signal = object()

    def stop(self):
        self._queue.put(self._shutdown_signal)
        self.join()

    def run(self):
        # Here wait on the queue until an data appears, or till the next
        # time to flush the writer, whichever is earlier. If we have an
        # data, write it. If not, an empty queue exception will be raised
        # and we can proceed to flush the writer.
        while True:
            now = time.time()
            queue_wait_duration = self._next_flush_time - now
            data = None
            try:
                if queue_wait_duration > 0:
                    data = self._queue.get(True, queue_wait_duration)
                else:
                    data = self._queue.get(False)

                if data is self._shutdown_signal:
                    return
                self._record_writer.write(data)
                self._has_pending_data = True
            except queue.Empty:
                pass
            finally:
                if data:
                    self._queue.task_done()

            now = time.time()
            if now > self._next_flush_time:
                if self._has_pending_data:
                    # Small optimization - if there are no pending data,
                    # there's no need to flush, since each flush can be
                    # expensive (e.g. uploading a new file to a server).
                    self._record_writer.flush()
                    self._has_pending_data = False
                # Do it again in flush_secs.
                self._next_flush_time = now + self._flush_secs



# Copyright 2020 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Base Class of Experiment Data Access API."""


import abc


class BaseExperiment(metaclass=abc.ABCMeta):
    """Base class for experiment data access."""

    # TODO(cais): Add list_scalar_runs().
    # TODO(cais): Add list_scalar_tags().

    @abc.abstractmethod
    def get_scalars(
        self,
        runs_filter=None,
        tags_filter=None,
        pivot=False,
        include_wall_time=False,
    ):
        """Export scalar data as a pandas.DataFrame.

        Args:
          runs_filter: A regex filter for runs (e.g., r'run_[2-4]'). Operates in
            logical AND relation with `tags_filter`.
          tags_filter: A regex filter for tags (e.g., r'.*loss.*'). Operates in
            logical AND related with `runs_filter`.
          pivot: Whether to returned DataFrame will be pivoted (via pandas’
            `pivot_data()` method to a “wide” format wherein the tags of a
            given run and a given step are all collected in a single row.
            Setting `pivot` to `True` stipulates that the sets of step values
            are identical among all tags in every run of the experiment (after
            any run and tag filtering), so that the pivoting operation will not
            introduce missing values in the resultant DataFrame. Failing to meet
            this condition will cause `pivot=True` to raise a `ValueError`.
            If not provided, defaults to `False`.
          include_wall_time: Include wall_time (timestamps in nanoseconds since
            the epoch in float64) as a column in the returned DataFrame.
            If not provided, defaults to `False`.

        Returns:
          If `pivot` (default):
            A pivoted DataFrame with the indexing columns of
              - run
              - step
            And value columns that correspond to the tags.
            Duplicate entries for each run-step combination will be aggregated
            with `numpy.stack`. This format is more friendly to manipulation and
            plotting and hence io chosen as the default. When certain rows have
            missing values, a warning message will be displayed and advise the
            user to use the `pivot=False` if steps have different meanings in
            the experiment.
          If `not pivot`:
            A DataFrame with the following columns.
              - run: (non-null object)
              - tag: (non-null object)
              - steps: (non-null int64)
              - wall_time: (non-null object)
              - value: (non-null float32)
        """
        # TODO(cais): Add description about sorting order.
        pass
